@@ -7,7 +7,7 @@
  * within the fixbuf structure
  *
  * ------------------------------------------------------------------------
- * Copyright (C) 2008-2014 Carnegie Mellon University. All Rights Reserved.
+ * Copyright (C) 2008-2015 Carnegie Mellon University. All Rights Reserved.
  * ------------------------------------------------------------------------
  * Authors: Chris Inacio <inacio@cert.org>, Emily Sarneso <ecoff@cert.org>
  * ------------------------------------------------------------------------
@@ -70,6 +70,8 @@
 
 #include "fbcollector.h"
 
+#include <pthread.h>
+
 #ident "$Id$"
 
 #ifndef FB_NETFLOW_DEBUG
@@ -277,6 +279,7 @@ struct fbCollectorNetflowV9State_st {
     fbCollectorNetflowV9Session_t *session;
     /* need to keep templates per domain */
     GHashTable                    *domainHash;
+    pthread_mutex_t               ts_lock;
 };
 
 /**
@@ -874,7 +877,7 @@ static int netflowOptionsTemplateParse (
     uintptr_t       bigTemplateId;
     uint16_t        recLength = g_ntohs(*recordLength);
     unsigned int    loop;
-    unsigned int    fieldCount = 0;
+    unsigned int    fieldCount;
     gpointer        hashResult = NULL;
     int             tmplcount = 0;
     struct fbCollectorNetflowV9State_st     *transState =
@@ -884,6 +887,8 @@ static int netflowOptionsTemplateParse (
 
     while (lengthParsed < recLength) {
 
+        fieldCount = 0;
+
         if ((recLength - lengthParsed) < 6) {
             /* not enough for an options header - probably extra padding */
             return tmplcount;
@@ -1116,6 +1121,8 @@ static gboolean     fbCollectorPostProcV9(
     /* read the observation domain */
     READU32INC(msgOsetPtr, obsDomain);
 
+    pthread_mutex_lock(&transState->ts_lock);
+
     transState->observation_id = obsDomain;
 
     if (transState->sessionptr != collector->udp_head->session) {
@@ -1199,6 +1206,7 @@ static gboolean     fbCollectorPostProcV9(
             g_set_error(err, FB_ERROR_DOMAIN, FB_ERROR_NETFLOWV9,
                         "Invalid Netflow %s Record Length (%u < 4)",
                         ((1 == setId) ? "Options" : "Data"), recordLength);
+            pthread_mutex_unlock(&transState->ts_lock);
             return FALSE;
         }
         /* Check to make sure we won't overrun buffer - Add 4 for set header */
@@ -1208,6 +1216,7 @@ static gboolean     fbCollectorPostProcV9(
                         "larger than remaining buffer length (%ld)",
                         (1 == setId) ? "Options" : "Record",
                         recordLength, ((dataBuf + *bufLen + 4) - msgOsetPtr));
+            pthread_mutex_unlock(&transState->ts_lock);
             return FALSE;
         }
 
@@ -1220,6 +1229,7 @@ static gboolean     fbCollectorPostProcV9(
                                                     recLengthPtr,
                                                     dataBuf, bufLen, err);
             if (!tmpls_parsed) {
+                pthread_mutex_unlock(&transState->ts_lock);
                 return FALSE;
             }
 
@@ -1241,6 +1251,7 @@ static gboolean     fbCollectorPostProcV9(
                                                        recLengthPtr, err);
             if (!tmpls_parsed) {
                 /* Needs to contain at least 1 */
+                pthread_mutex_unlock(&transState->ts_lock);
                 return FALSE;
             }
 
@@ -1255,6 +1266,7 @@ static gboolean     fbCollectorPostProcV9(
             /* data records must be 256 or higher */
             g_set_error(err, FB_ERROR_DOMAIN, FB_ERROR_NETFLOWV9,
                         "NetFlow record type (%u) is not supported", setId);
+            pthread_mutex_unlock(&transState->ts_lock);
             return FALSE;
 
         } else {
@@ -1270,6 +1282,7 @@ static gboolean     fbCollectorPostProcV9(
                                 "No Templates Present for this session."
                                 " %u Flows Lost.", recordCount-recordCounter);
                     currentSession->netflowSeqNum++;
+                    pthread_mutex_unlock(&transState->ts_lock);
                     return FALSE;
                 }
                 /* else, remove these bytes from the packet */
@@ -1296,6 +1309,7 @@ static gboolean     fbCollectorPostProcV9(
                                 " %u Flows Lost.", setId,
                                 (recordCount-recordCounter));
                     currentSession->netflowSeqNum++;
+                    pthread_mutex_unlock(&transState->ts_lock);
                     return FALSE;
                 }
                 /* else, remove these bytes from the packet */
@@ -1326,6 +1340,7 @@ static gboolean     fbCollectorPostProcV9(
                 if (numberRecordsInSet == 0) {
                     g_set_error(err, FB_ERROR_DOMAIN, FB_ERROR_NETFLOWV9,
                                 "NetFlow Data Record with 0 Records");
+                    pthread_mutex_unlock(&transState->ts_lock);
                     return FALSE;
                 }
 
@@ -1342,6 +1357,7 @@ static gboolean     fbCollectorPostProcV9(
                                         "NetFlow V9 unable to convert "
                                         "information model "
                                         "time elements, no space");
+                            pthread_mutex_unlock(&transState->ts_lock);
                             return FALSE;
                         }
 
@@ -1375,6 +1391,7 @@ static gboolean     fbCollectorPostProcV9(
                     "%u, processed %u)", (unsigned int)(*bufLen),
                     ntohs(*lengthCountPtr));
         currentSession->netflowSeqNum++;
+        pthread_mutex_unlock(&transState->ts_lock);
         return FALSE;
     }
 
@@ -1414,6 +1431,8 @@ static gboolean     fbCollectorPostProcV9(
     }
 #endif
 
+    pthread_mutex_unlock(&transState->ts_lock);
+
     return TRUE;
 }
 
@@ -1440,6 +1459,8 @@ static void         fbCollectorTransCloseV9(
     g_hash_table_destroy(transState->domainHash);
     transState->domainHash = NULL;
 
+    pthread_mutex_destroy(&transState->ts_lock);
+
     if (NULL != collector->translatorState) {
         g_free(collector->translatorState);
     }
@@ -1470,9 +1491,12 @@ static void fbCollectorTimeOutSessionV9(
         return;
     }
 
+    pthread_mutex_lock(&transState->ts_lock);
+
     nfsession = g_hash_table_lookup(transState->domainHash, session);
     if (nfsession == NULL) {
         /* don't need to free! */
+        pthread_mutex_unlock(&transState->ts_lock);
         return;
     }
 
@@ -1484,6 +1508,8 @@ static void fbCollectorTimeOutSessionV9(
         transState->session = NULL;
     }
 
+    pthread_mutex_unlock(&transState->ts_lock);
+
 }
 
 
@@ -1537,6 +1563,7 @@ gboolean    fbCollectorSetNetflowV9Translator(
     nflowState->domainHash = hashTable;
     nflowState->observation_id = 0;
     nflowState->sessionptr = NULL;
+    pthread_mutex_init(&nflowState->ts_lock, NULL);
 
     return fbCollectorSetTranslator(collector, fbCollectorPostProcV9,
         fbCollectorDecodeV9MsgVL, fbCollectorMessageHeaderV9,
@@ -1574,6 +1601,8 @@ uint32_t fbCollectorGetNetflowMissed(
     struct fbCollectorNetflowV9State_st     *ts = NULL;
     fbUDPConnSpec_t                         *udp = NULL;
     fbSession_t                             *session = NULL;
+    fbCollectorNetflowV9Session_t           *ts_session = NULL;
+    uint32_t                                missed = 0;
 
     if (!collector) {
         return 0;
@@ -1610,14 +1639,20 @@ uint32_t fbCollectorGetNetflowMissed(
         return 0;
     }
 
+    pthread_mutex_lock(&ts->ts_lock);
+
     if (ts->sessionptr != session) {
         /* lookup template Hash Table per Domain */
-        ts->session = g_hash_table_lookup(ts->domainHash, session);
-        if (ts->session == NULL) {
-            return 0;
-        }
+        ts_session = g_hash_table_lookup(ts->domainHash, session);
+    } else {
+        ts_session = ts->session;
     }
 
-    return ts->session->netflowMissed;
+    if (ts_session) {
+        missed = ts_session->netflowMissed;
+    }
 
+    pthread_mutex_unlock(&ts->ts_lock);
+
+    return missed;
 }
